@@ -1,6 +1,6 @@
 
 from megrok import pagetemplate as pt
-from grokcore.viewlet.util import sort_components
+from grokcore.component.util import sort_components
 from grokcore import component as grok
 from zope import component
 
@@ -45,7 +45,7 @@ class SubFormGroupBase(object):
                 (self.context, self,  self.request), interfaces.ISubForm))
         # sort them
         self.allSubforms = sort_components(subforms)
-        self.subforms = self._getAvailableSubForms()
+        self.subforms = []
 
     def getSubForm(self, identifier):
         for form in self.subforms:
@@ -66,18 +66,18 @@ class SubFormGroupBase(object):
 
     def updateActions(self):
         # Set/run actions for all forms
-        action, status = None, None
+        form, action, status = self, None, None
         for subform in self._getAvailableSubForms():
-            action, status = subform.updateActions()
+            form, action, status = subform.updateActions()
             if action is not None:
                 break
-        # The result of the actions might have changed the available subforms
-        self.subforms = self._getAvailableSubForms()
-        return action, status
+        return form, action, status
 
     def updateWidgets(self):
+        # Set the subforms to render
+        self.subforms = self._getAvailableSubForms()
         # Set widgets for all forms
-        for subform in self._getAvailableSubForms():
+        for subform in self.subforms:
             subform.updateWidgets()
 
     def _getAvailableSubForms(self):
@@ -106,7 +106,7 @@ class SubFormGroup(SubFormBase, SubFormGroupBase, form.GrokViewSupport):
     grok.implements(interfaces.ISubForm)
 
     def available(self):
-        return len(self.subforms) != 0
+        return len(self._getAvailableSubForms()) != 0
 
 
 class SubFormGroupTemplate(pt.PageTemplate):
@@ -122,7 +122,7 @@ class ComposedForm(SubFormGroupBase, form.Form):
     grok.implements(interfaces.IComposedForm)
 
     def updateForm(self):
-        action, status = SubFormGroupBase.updateActions(self)
+        executed_form, action, status = SubFormGroupBase.updateActions(self)
         if action is None:
             form.Form.updateActions(self)
         SubFormGroupBase.updateWidgets(self)
